<?php

class MAX_CssMin
	{
	static public function run($sCss)
		{
		
		/*
			Mise en forme
		*/
		
		// On retire les commentaires
		$sCss = self::stripComments($sCss);
		
		// Retrait des sauts de lignes
		$sCss = preg_replace("`\n+`", "\n", $sCss);
		
		// Retrait des espaces en début et fin de ligne
		$sCss = preg_replace("`^\s+`m", "", $sCss);
		$sCss = preg_replace("`\s+$`m", "", $sCss);
		$sCss = preg_replace("`^\s+`", "", $sCss); // Espaces en début de fichier
		
		// Espaces "en double"
		$sCss = preg_replace("`[ \t]+`", " ", $sCss);
		
		// Espaces inutiles et mise en une ligne
		$sCss = preg_replace("`(?<=[;:,{])\s+`", "", $sCss);
		
		// Espaces inutiles 2
		$sCss = preg_replace("`\s+(?=[:,;{}])`", "", $sCss);
		$sCss = preg_replace("`\s+(?=[!]important)`", "", $sCss);
		
		// Espaces inutiles dans les déclarations (ex:  a > b  =>  a>b)
		$sCss = str_replace(' > ', '>', $sCss);
		
		/*
			Amélioration
		*/
		
		// Avant tous, on corrige d'éventuels derniers points virgules manquants (On le fait avant, car les autres régles dépendent du ;)
		$sCss = preg_replace("`(?<![;])}`", ";}", $sCss);
		
		// Les 0px ou 0% en 0
		$sCss = preg_replace("`(?<=[: ]0)(px|em|ex|cm|mm|in|pt|pc|%)`", "", $sCss);
		
		// 0.5 en .5
		$sCss = preg_replace("`(?<=[: ])0(?=\.[0-9]+(px|em|ex|cm|mm|in|pt|pc|%)?[;! ])`", "", $sCss);
		
		// 10px 20px 10px 20px => 10px 20px
		$sCss = self::minMultipleWidthValues($sCss);
		
		
		// Diminution des couleurs rgb(x,y,z) en #xxyyzz
		$sCss = preg_replace_callback('`rgb\((-?[0-9]+%?),(-?[0-9]+%?),(-?[0-9]+%?)\)`', array(get_class(), 'convertColorsRgbToHexCallback'), $sCss);
		
		// Mise en lowercase des couleurs Héxa
		$sCss = preg_replace_callback('`(?<=#)[0-9a-fA-F]{3,6}`', array(get_class(), 'convertColorsHexToLowercaseCallback'), $sCss);
		
		// Diminution des couleurs #xxyyzz en #xyz
		$sCss = preg_replace('`(?<=#)([0-9a-fA-F])\1([0-9a-fA-F])\2([0-9a-fA-F])\3`', "$1$2$3", $sCss);
		
		// Couleurs "spéciales" (voir : http://www.w3.org/TR/CSS2/syndata.html#value-def-color )
		$aColorsCss = array(	'maroon'=>'#800000',	'red'=>'#f00',		'orange'=>'#ffA500',	'yellow'=>'#ff0',
					'olive'=>'#808000',	'purple'=>'#800080',	'fuchsia'=>'#f0f',	'white'=>'#fff',
					'lime'=>'#0f0',		'green'=>'#008000',	'navy'=>'#000080',	'blue'=>'#00f',
					'aqua'=>'#0ff',		'teal'=>'#008080',	'black'=>'#000',	'silver'=>'#c0c0c0',
					'gray'=>'#808080');
		foreach ($aColorsCss as $sName => $sHexVal)
			{
			$iNameLen = strlen($sName);
			$iHexValLen = strlen($sHexVal);
			if ($iNameLen != $iHexValLen)
				{
				if ($iNameLen > $iHexValLen)
					{
					$sCss = preg_replace("`(?<=[: ])".$sName."(?=[;! ])`", $sHexVal, $sCss);
					}
				else
					{
					$sCss = preg_replace("`(?<=[: ])".$sHexVal."(?=[;! ])`", $sName, $sCss);
					}
				}
			}
		
		// Fontweight
		$sCss = str_replace('font-weight:normal', 'font-weight:400', $sCss);
		$sCss = str_replace('font-weight:bold', 'font-weight:700', $sCss);
		
		// URI : url( 'bla/bla.jpg' ) => url(bla/bla.jpg)
		$sCss = self::minUri($sCss);
		
		// Background positions
		$motif = "(?:top|center|bottom|left|right|-?[\d.]+(?:px|em|ex|cm|mm|in|pt|pc|%)?)";
		$sCss = preg_replace_callback("`(?<=background-position:)(".$motif."(?: ".$motif.")?)(?=;)`", array(get_class(), 'minBgPosValuesCallback'), $sCss);
		$sCss = preg_replace_callback("`(?<=background:)([^;]*?)( ".$motif."(?: ".$motif.")?)`", array(get_class(), 'minBgPosValuesCallback2'), $sCss);
		
		// Background property
		// Given a valid declaration, the 'background' property first sets all the individual background properties to their 
		// initial values, then assigns explicit values given in the declaration. 
		// See : http://www.w3.org/TR/CSS2/colors.html#propdef-background 
		$sCss = preg_replace_callback("`(?<=background:)[^;!]*(?=[;!])`", array(get_class(), 'minBgPropertyCallback'), $sCss);

		/*
			Fin des mises en forme
		*/
		
		// Derniers points virgules inutiles (On le fait à la fin)
		$sCss = preg_replace("`;}`", "}", $sCss);
		
		// Efface les retours à la ligne (à désactiver par défaut)
		//$sCss = preg_replace("`\n`", "", $sCss);
		
		/*
		// Suppression des propriétés en double (même nom, même valeur)
		$sCss = preg_replace_callback("`(?<=[{])[^{}]+(?=[}])`", array(get_class(), 'test'), $sCss);
		*/
		
		return $sCss;
		}
	
	/*
	static public function test($aMatch)
		{
		$aProperties = explode(';', $aMatch[0]);
		$aProperties = array_unique($aProperties);
		return implode(';', $aProperties);
		}
	*/	
	
	static public function stripComments($sCss)
		{
		// On retire les commentaires
		
		// Commentaires  /* blabla */
		$sCss = preg_replace('`/\*.*\*/`sU', '', $sCss);
		
		return $sCss;
		}
	
	static public function minMultipleWidthValues($sCss)
		{
		// Diminution des tailles combinées (on les fait de 4=>3, 3=>2, 2=>1 dans cet ordre pour passer récursivement 4=>3=>2=>1 si possible)
		// Doit être fait que pour (margin|padding|border-width)
		// On égalise le nb de lettre dans le ( | | ) du lookbehind, car sinon erreur.
		
		$motif = '(-?[\d.]+(?:px|em|ex|cm|mm|in|pt|pc|%)?|auto)';
		
		// x y z y => x y z
		$sCss = preg_replace('`(?<=(?:margin|adding|-width):)'.$motif.' '.$motif.' '.$motif.' \2(?=[;!])`', "$1 $2 $3", $sCss);
		// x y x => x y
		$sCss = preg_replace('`(?<=(?:margin|adding|-width):)'.$motif.' '.$motif.' \1(?=[;!])`', "$1 $2", $sCss);
		// x x => x
		$sCss = preg_replace('`(?<=(?:margin|adding|-width):)'.$motif.' \1(?=[;!])`', "$1", $sCss);
		
		return $sCss;
		}
	
	static public function minUri($sCss)
		{
		// url('bla/bla.jpg') => url(bla/bla.jpg)
		// Uniquement si pas de caractères spéciaux pour un url()
		// Voir : http://www.w3.org/TR/CSS21/syndata.html#uri
		
		$sCss = preg_replace('`url\( ?([\'"])([a-zA-Z0-9._/:-]+)\1 ?\)`', 'url($2)', $sCss);
		
		return $sCss;
		}
	
	static public function convertColorsRgbToHexCallback($aMatch)
		{
		// $sRgb est de la forme rgb(255,0,0)
		$sHex = '#';
		for ($i=1; $i<=3; $i++)
			{
			$sVal = $aMatch[$i];
			$iVal = intval($sVal);
			if (substr($sVal, -1, 1)=='%')
				{
				$iVal = round($iVal * 255 / 100);
				}
		
			if ($iVal > 255)
				{ $iVal = 255; }
			elseif ($iVal < 0)
				{ $iVal = 0; }
		
			$sHexVal = dechex($iVal);
			if (strlen($sHexVal)==1)
				{ $sHexVal = '0'.$sHexVal; }
		
			$sHex .= $sHexVal;
			}
		return $sHex;
		}
	
	static public function convertColorsHexToLowercaseCallback($aMatch)
		{
		return strtolower($aMatch[0]);
		}

	static public function minBgPosValuesCallback($aMatch)
		{
		return self::minBgPosValues($aMatch[1]);
		}
	static public function minBgPosValuesCallback2($aMatch)
		{
		$sPos = self::minBgPosValues($aMatch[2]);
		
		return $aMatch[1].' '.$sPos;
		}
	static public function minBgPosValues($sPos)
		{
		$aPos = explode(' ', trim($sPos));
		$sVal1 = $aPos[0];
		$sVal2 = isset($aPos[1]) ? $aPos[1] : 'center';
		$aKeywordsX = array('left' => '0', 'center' => '50%', 'right' => '100%');
		$aKeywordsY = array('top' => '0', 'center' => '50%', 'bottom' => '100%');
		
		$bInvert = self::toolBgPosHasToInvert($sVal1, $sVal2);
		$sValX = ($bInvert) ? $sVal2 : $sVal1;
		$sValY = ($bInvert) ? $sVal1 : $sVal2;
		
		// On positionne 0 => top .... pour la suite (réduit les cas : ex : "50% ou center" ...)
		if (in_array($sValX, $aKeywordsX))
			{ $sValX = array_search($sValX, $aKeywordsX); }
		if (in_array($sValY, $aKeywordsY))
			{ $sValY = array_search($sValY, $aKeywordsY); }
	
		if (!array_key_exists($sValX, $aKeywordsX) && !array_key_exists($sValY, $aKeywordsY))
			{
			return $sValX.' '.$sValY;
			}
		
		if ($sValY=='center')
			{
			if (array_key_exists($sValX, $aKeywordsX))
				{
				return $aKeywordsX[$sValX];
				}
			return $sValX;
			}
		if ($sValX=="center" && array_key_exists($sValY, $aKeywordsY))
			{
			return $sValY;
			}
		
		if (array_key_exists($sValX, $aKeywordsX))
			{
			$sValX = $aKeywordsX[$sValX];
			}
		if (array_key_exists($sValY, $aKeywordsY))
			{
			$sValY = $aKeywordsY[$sValY];
			}
		
		return $sValX.' '.$sValY;
		}
	
	static public function toolBgPosHasToInvert($sVal1, $sVal2)
		{
		if (in_array($sVal1, array('top', 'bottom')))
			{
			return true;
			}
		if (in_array($sVal2, array('left', 'right')))
			{
			return true;
			}
		
		return false;
		}
	
	static public function minBgPropertyCallback($aMatch)
		{
		// Enlève les valeurs initiales chacune des propriétés séparées
		$sProperties = ' '.$aMatch[0].' ';
		
		// background-color : transparent
		$sProperties = str_replace(' transparent ', ' ', $sProperties);
		
		// background-image : none
		$sProperties = str_replace(' none ', ' ', $sProperties);
		
		// background-repeat : repeat
		$sProperties = str_replace(' repeat ', ' ', $sProperties);
		
		// background-attachment : scroll
		$sProperties = str_replace(' scroll ', ' ', $sProperties);
		
		// background-position : 0 0
		$sProperties = str_replace(' 0 0 ', ' ', $sProperties);
		
		$sProperties = trim($sProperties);
		
		// Si il y a plus rien, on met la plus courte
		if ($sProperties=='')
			{
			// On pourrait renvoyer '0 0', on gagnerait un caractère
			// On renvoit pas '0' car ça reviendrait à "left center", différent du "left top" mis initialement
			return 'none';
			}
		
		return $sProperties;
		}
	}

?>
